﻿Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=3
@EndOfDesignText@
'Class module
Sub Class_Globals

	Type Coordinate( _
		Latitude As Double, _
		Longitude As Double _
	)

	Type CountryBoundary( _
		Continent As String, _
		NameLong As String _
	)

	Private DATABASE_FILENAME As String = "ne_110m_admin_0_countries.sqlite"
	Private DATABASE_FILENAME2 As String = "TestDB.db3"
	Private DATABASE_PATH As String=File.DirDefaultExternal
	'	SRID is the 'spatial reference id', a value of 4326 corresponds to WGS84 projection
	Private SRID As Int = 4326
	Private TABLE_NAME As String = "ne_110m_admin_0_countries"
	
	Private SQL1 As SQLCipher
	Private SpatialiteConstants As Spatialite_Constants
	Private SpatialiteDatabase As Spatialite_Database

End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize
	
	If Not(File.Exists(DATABASE_PATH, DATABASE_FILENAME2)) Then
		File.Copy(File.DirAssets, DATABASE_FILENAME2, DATABASE_PATH, DATABASE_FILENAME2)
	End If
	
	'non-spatialite DB, accessed by SQLCipher 1.60, note that there needs to be reference to the regular SQL library
	'although there will be warning that the SQL library is not used
	'---------------------------------------------------------------------------------------------------------------
	SQL1.Initialize(DATABASE_PATH, DATABASE_FILENAME2, False, "" , "")
	
	If Not(File.Exists(DATABASE_PATH, DATABASE_FILENAME)) Then
		File.Copy(File.DirAssets, DATABASE_FILENAME, DATABASE_PATH, DATABASE_FILENAME)
	End If
	
	'spatialite DB accessed by Spatialite library 1.10
	Try 'can't catch this error
		SpatialiteDatabase.Initialize '<<<<< error here with SQLCipher 1.60, fine with SQLCipher 1.50
		'java.lang.UnsatisfiedLinkError: No implementation found for void jsqlite.Database.internal_init() (tried Java_jsqlite_Database_internal_1init and Java_jsqlite_Database_internal_1init__)
	Catch
		Log("CountryFinder.Initialize, error: " & GetErrorString)
		Return
	End Try
	
	SpatialiteDatabase.Open(DATABASE_PATH, DATABASE_FILENAME, SpatialiteConstants.SQLITE_OPEN_READONLY)
	
End Sub

Sub GetErrorString() As String
	
	Dim strError As String
	Dim iPos As Int

	If LastException.IsInitialized = False Then
		Return "No error"
	Else
		strError = LastException.Message
		'RunLog("GetErrorString, strError: " & strError)
		iPos = strError.IndexOf("###")
		If iPos > -1 And iPos < strError.Length - 1 Then
			strError = strError.SubString2(0, iPos)
		End If
	End If
	
	Return strError

End Sub

Public Sub Close
	SpatialiteDatabase.Close
End Sub

Public Sub GetCountryBoundarys(Coordinate1 As Coordinate) As List
	'	queries the spatialite database for the CountryBoundarys that contains Latitude, Longitude
	'	returns a List of CountryBoundarys
	'	the List will not be initialized if no CountryBoundarys contains Latitude, Longitude
	'	if more than one CountryBoundary contains Latitude, Longitude then the List will be sorted alphabetically by long name
	Dim ResultSet As List	'	will return a List of CountryBoundary types
	
	Dim SpatialiteStatement As Spatialite_Stmt
	SpatialiteStatement=SpatialiteDatabase.Prepare("SELECT name_long, continent FROM "&TABLE_NAME&" WHERE ST_Within(MakePoint(?, ?, ?), Geometry) ORDER BY name_long ASC;")
	SpatialiteStatement.BindDouble(1, Coordinate1.Longitude)
	SpatialiteStatement.BindDouble(2, Coordinate1.Latitude)
	SpatialiteStatement.BindInt(3, SRID)
	
	Do While SpatialiteStatement.Step
		Dim CountryBoundary1 As CountryBoundary
		CountryBoundary1.Initialize
		CountryBoundary1.NameLong=SpatialiteStatement.ColumnString(0)
		CountryBoundary1.Continent=SpatialiteStatement.ColumnString(1)
		If Not(ResultSet.IsInitialized) Then
			ResultSet.Initialize
		End If
		ResultSet.Add(CountryBoundary1)
	Loop
	
	SpatialiteStatement.Close
	
	Return ResultSet
End Sub

